@@ -5,19 +5,31 @@ from trytond.model import ModelView, ModelSQL, ValueMixin, fields
 from trytond.pool import Pool, PoolMeta
 from trytond.tools.multivalue import migrate_property
 
-__all__ = ['Address', 'Party', 'PartyPaymentTerm', 'PartyReplace']
+__all__ = ['Address', 'Party', 'PartyPaymentTerm',
+    'PartyReplace', 'PartyErase']
 customer_payment_term = fields.Many2One(
     'account.invoice.payment_term', "Customer Payment Term")
 supplier_payment_term = fields.Many2One(
     'account.invoice.payment_term', "Supplier Payment Term")
 
 
-class Address:
-    __metaclass__ = PoolMeta
+class Address(metaclass=PoolMeta):
     __name__ = 'party.address'
     invoice = fields.Boolean('Invoice')
 
 
+class ContactMechanism(metaclass=PoolMeta):
+    __name__ = 'party.contact_mechanism'
+    invoice = fields.Boolean('Invoice')
+
+    @classmethod
+    def usages(cls, _fields=None):
+        if _fields is None:
+            _fields = []
+        _fields.append('invoice')
+        return super(ContactMechanism, cls).usages(_fields=_fields)
+
+
 class Party(ModelSQL, ModelView):
     __name__ = 'party.party'
     customer_payment_term = fields.MultiValue(customer_payment_term)
@@ -25,23 +37,6 @@ class Party(ModelSQL, ModelView):
     payment_terms = fields.One2Many(
         'party.party.payment_term', 'party', "Payment Terms")
 
-    @classmethod
-    def __register__(cls, module_name):
-        ModelField = Pool().get('ir.model.field')
-
-        # Migration from 2.2: property field payment_term renamed
-        # to customer_payment_term
-        fields = ModelField.search([
-                ('name', '=', 'payment_term'),
-                ('model.model', '=', 'party.party')
-                ])
-        if fields:
-            ModelField.write(fields, {
-                    'name': 'customer_payment_term',
-                    })
-
-        super(Party, cls).__register__(module_name)
-
     @classmethod
     def multivalue_model(cls, field):
         pool = Pool()
@@ -77,8 +72,7 @@ class PartyPaymentTerm(ModelSQL, ValueMixin):
             parent='party', fields=fields)
 
 
-class PartyReplace:
-    __metaclass__ = PoolMeta
+class PartyReplace(metaclass=PoolMeta):
     __name__ = 'party.replace'
 
     @classmethod
@@ -87,3 +81,32 @@ class PartyReplace:
             ('account.invoice', 'party'),
             ('account.invoice.line', 'party'),
             ]
+
+
+class PartyErase(metaclass=PoolMeta):
+    __name__ = 'party.erase'
+
+    @classmethod
+    def __setup__(cls):
+        super(PartyErase, cls).__setup__()
+        cls._error_messages.update({
+                'pending_invoice': (
+                    'The party "%(party)s" can not be erased '
+                    'because he has pending invoices '
+                    'for the company "%(company)s".'),
+                })
+
+    def check_erase_company(self, party, company):
+        pool = Pool()
+        Invoice = pool.get('account.invoice')
+        super(PartyErase, self).check_erase_company(party, company)
+
+        invoices = Invoice.search([
+                ('party', '=', party.id),
+                ('state', 'not in', ['paid', 'cancel']),
+                ])
+        if invoices:
+            self.raise_user_error('pending_invoice', {
+                    'party': party.rec_name,
+                    'company': company.rec_name,
+                    })
